@@ -2,10 +2,10 @@
 # -*- coding:utf-8 -*-
 # @Author: Jialiang Shi
 from packaging.version import parse
-from gerrit.changes.reviewers import Reviewers
-from gerrit.changes.revision import Revision
-from gerrit.changes.edit import Edit
-from gerrit.changes.messages import Messages
+from gerrit.changes.reviewers import GerritChangeReviewers
+from gerrit.changes.revision import GerritChangeRevision
+from gerrit.changes.edit import GerritChangeEdit
+from gerrit.changes.messages import GerritChangeMessages
 from gerrit.utils.models import BaseModel
 from gerrit.utils.exceptions import UnsupportMethod
 
@@ -13,27 +13,8 @@ from gerrit.utils.exceptions import UnsupportMethod
 class GerritChange(BaseModel):
     def __init__(self, **kwargs):
         super(GerritChange, self).__init__(**kwargs)
-        self.attributes = [
-            "id",
-            "project",
-            "branch",
-            "attention_set",
-            "change_id",
-            "subject",
-            "status",
-            "created",
-            "updated",
-            "submitted",
-            "mergeable",
-            "insertions",
-            "deletions",
-            "current_revision",
-            "_number",
-            "owner",
-            "gerrit",
-        ]
 
-    def update(self, input_):
+    def create_merge_patch_set(self, input_):
         """
         Update an existing change by using a MergePatchSetInput entity.
         Gerrit will create a merge commit based on the information of MergePatchSetInput and add a new patch set to
@@ -48,7 +29,7 @@ class GerritChange(BaseModel):
                 }
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.update(input_)
 
         :param input_: the MergePatchSetInput entity,
@@ -61,7 +42,7 @@ class GerritChange(BaseModel):
             base_url, json=input_, headers=self.gerrit.default_headers
         )
         result = self.gerrit.decode_response(response)
-        return self.gerrit.changes.get(result.get("id"))
+        return result
 
     def set_commit_message(self, input_):
         """
@@ -73,7 +54,7 @@ class GerritChange(BaseModel):
                 "message": "New Commit message \\n\\nChange-Id: I10394472cbd17dd12454f229e4f6de00b143a444\\n"
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.set_commit_message(input_)
 
         :param input_: the CommitMessageInput entity,
@@ -88,8 +69,7 @@ class GerritChange(BaseModel):
         result = self.gerrit.decode_response(response)
         return result
 
-    @property
-    def topic(self):
+    def get_topic(self):
         """
         Retrieves the topic of a change.
 
@@ -104,8 +84,7 @@ class GerritChange(BaseModel):
         result = self.gerrit.decode_response(response)
         return result
 
-    @topic.setter
-    def topic(self, topic):
+    def set_topic(self, topic):
         """
         Sets the topic of a change.
 
@@ -121,8 +100,7 @@ class GerritChange(BaseModel):
         result = self.gerrit.decode_response(response)
         return result
 
-    @topic.deleter
-    def topic(self):
+    def delete_topic(self):
         """
         Deletes the topic of a change.
 
@@ -140,7 +118,9 @@ class GerritChange(BaseModel):
         endpoint = "/changes/%s/assignee" % self.id
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        return self.gerrit.accounts.get(result.get("username"))
+        if result:
+            username = result.get("username")
+            return self.gerrit.accounts.get(username)
 
     def set_assignee(self, input_):
         """
@@ -152,7 +132,7 @@ class GerritChange(BaseModel):
                 "assignee": "jhon.doe"
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.set_assignee(input_)
 
         :param input_: the AssigneeInput entity,
@@ -165,7 +145,9 @@ class GerritChange(BaseModel):
             base_url, json=input_, headers=self.gerrit.default_headers
         )
         result = self.gerrit.decode_response(response)
-        return self.gerrit.accounts.get(result.get("username"))
+        if result:
+            username = result.get("username")
+            return self.gerrit.accounts.get(username)
 
     def get_past_assignees(self):
         """
@@ -176,7 +158,13 @@ class GerritChange(BaseModel):
         endpoint = "/changes/%s/past_assignees" % self.id
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        assignees = [self.gerrit.accounts.get(item.get("username")) for item in result]
+        assignees = []
+        if result:
+            for item in result:
+                username = item.get("username")
+                assignee = self.gerrit.accounts.get(username)
+                assignees.append(assignee)
+
         return assignees
 
     def delete_assignee(self):
@@ -188,8 +176,10 @@ class GerritChange(BaseModel):
         endpoint = "/changes/%s/assignee" % self.id
         response = self.gerrit.requester.delete(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
+
         if result:
-            return self.gerrit.accounts.get(result.get("username"))
+            username = result.get("username")
+            return self.gerrit.accounts.get(username)
 
     def get_pure_revert(self, commit):
         """
@@ -207,24 +197,28 @@ class GerritChange(BaseModel):
         """
         Abandons a change.
         Abandoning a change also removes all users from the attention set.
+        If the change cannot be abandoned because the change state doesn’t allow abandoning of the change,
+        the response is “409 Conflict” and the error message is contained in the response body.
 
         :return:
         """
         endpoint = "/changes/%s/abandon" % self.id
         response = self.gerrit.requester.post(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        return self.gerrit.changes.get(result.get("id"))
+        return result
 
     def restore(self):
         """
         Restores a change.
+        If the change cannot be restored because the change state doesn’t allow restoring the change,
+        the response is “409 Conflict” and the error message is contained in the response body.
 
         :return:
         """
         endpoint = "/changes/%s/restore" % self.id
         response = self.gerrit.requester.post(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        return self.gerrit.changes.get(result.get("id"))
+        return result
 
     def rebase(self, input_):
         """
@@ -238,7 +232,7 @@ class GerritChange(BaseModel):
                 "base" : "1234",
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.rebase(input_)
 
         :param input_: the RebaseInput entity,
@@ -251,7 +245,7 @@ class GerritChange(BaseModel):
             base_url, json=input_, headers=self.gerrit.default_headers
         )
         result = self.gerrit.decode_response(response)
-        return self.gerrit.changes.get(result.get("id"))
+        return result
 
     def move(self, input_):
         """
@@ -265,7 +259,7 @@ class GerritChange(BaseModel):
                 "destination_branch" : "release-branch"
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.move(input_)
 
         :param input_: the MoveInput entity,
@@ -278,7 +272,7 @@ class GerritChange(BaseModel):
             base_url, json=input_, headers=self.gerrit.default_headers
         )
         result = self.gerrit.decode_response(response)
-        return self.gerrit.changes.get(result.get("id"))
+        return result
 
     def revert(self, input_=None):
         """
@@ -297,7 +291,7 @@ class GerritChange(BaseModel):
                 "message" : "Message to be added as review comment to the change when reverting the change."
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.revert()
             # or
             result = change.revert(input_)
@@ -312,7 +306,7 @@ class GerritChange(BaseModel):
             base_url, json=input_ or {}, headers=self.gerrit.default_headers
         )
         result = self.gerrit.decode_response(response)
-        return self.gerrit.changes.get(result.get("id"))
+        return result
 
     def revert_submission(self):
         """
@@ -349,7 +343,7 @@ class GerritChange(BaseModel):
                 "on_behalf_of": 1001439
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.submit(input_)
 
         :param input_: the SubmitInput entity,
@@ -362,7 +356,7 @@ class GerritChange(BaseModel):
             base_url, json=input_, headers=self.gerrit.default_headers
         )
         result = self.gerrit.decode_response(response)
-        return self.gerrit.changes.get(result.get("id"))
+        return result
 
     def delete(self):
         """
@@ -452,7 +446,7 @@ class GerritChange(BaseModel):
                 "expect_merged_as": "something"
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.fix()
             # or
             result = change.fix(input_)
@@ -486,7 +480,7 @@ class GerritChange(BaseModel):
                 "message": "Refactoring needs to be done before we can proceed here."
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.set_work_in_progress(input_)
             # or
             result = change.set_work_in_progress()
@@ -513,7 +507,7 @@ class GerritChange(BaseModel):
                 'message': 'Refactoring is done.'
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             change.set_ready_for_review(input_)
 
         :param input_: the WorkInProgressInput entity,
@@ -536,7 +530,7 @@ class GerritChange(BaseModel):
             input_ = {
                 "message": "After this security fix has been released we can make it public now."
             }
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             change.mark_private(input_)
 
         :param input_: the PrivateInput entity,
@@ -559,7 +553,7 @@ class GerritChange(BaseModel):
             input_ = {
                 "message": "This is a security fix that must not be public."
             }
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             change.unmark_private(input_)
             # or
             change.unmark_private()
@@ -641,7 +635,7 @@ class GerritChange(BaseModel):
                     "hashtag2"
                 ]
             }
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.set_hashtags(input_)
 
         :param input_: the HashtagsInput entity,
@@ -658,7 +652,7 @@ class GerritChange(BaseModel):
 
     @property
     def messages(self):
-        return Messages(change=self.id, gerrit=self.gerrit)
+        return GerritChangeMessages(change=self.id, gerrit=self.gerrit)
 
     def get_edit(self):
         """
@@ -671,8 +665,7 @@ class GerritChange(BaseModel):
         endpoint = "/changes/%s/edit" % self.id
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        if result:
-            return Edit.parse(result, change=self.id, gerrit=self.gerrit)
+        return GerritChangeEdit.parse(result, change=self.id, gerrit=self.gerrit)
 
     def create_empty_edit(self):
         """
@@ -685,7 +678,7 @@ class GerritChange(BaseModel):
 
     @property
     def reviewers(self):
-        return Reviewers(change=self.id, gerrit=self.gerrit)
+        return GerritChangeReviewers(change=self.id, gerrit=self.gerrit)
 
     def get_revision(self, revision_id):
         """
@@ -694,7 +687,7 @@ class GerritChange(BaseModel):
         :param revision_id:
         :return:
         """
-        return Revision(
+        return GerritChangeRevision(
             project=self.project,
             change=self.id,
             revision=revision_id,
@@ -731,7 +724,7 @@ class GerritChange(BaseModel):
                 "user": "John Doe",
                 "reason": "reason"
             }
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             result = change.add_to_attention_set(input_)
 
         :param input_: the AttentionSetInput entity,
@@ -763,7 +756,7 @@ class GerritChange(BaseModel):
             input_ = {
                 "reason": "reason"
             }
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             change.remove_from_attention_set('kevin.shi', input_)
             # or
             change.remove_from_attention_set('kevin.shi')
